<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Adapter;

use Exception;
use FiloBlu\MappConnector\Helper\FileLoggerHelper;
use Magento\Framework\DataObject as DataObject;
use Magento\Framework\HTTP\ClientInterface;
use Magento\Framework\HTTP\ClientInterfaceFactory;

/**
 * Class RestAdapter
 * @package FiloBlu\MappConnector\Adapter
 */
class RestAdapter implements MappAdapterInterface
{
    /**
     * Rest Config
     *
     * @var
     */
    protected $_config = [];

    /**
     * System Config
     *
     * @var
     */
    protected $_systemconfig;

    /**
     * FileLogger Helper
     *
     * @var FileLoggerHelper
     */
    protected $_fileLogger;
    /**
     * @var ClientInterfaceFactory
     */
    private $curlClientFactory;

    /**
     * RestAdapter constructor.
     * @param ClientInterfaceFactory $curlClientFactory
     */
    public function __construct(ClientInterfaceFactory $curlClientFactory)
    {
        $this->curlClientFactory = $curlClientFactory;
    }

    /**
     * Inits the Adapter
     *
     */
    public function _init()
    {
    }

    /**
     * Gets the Adapter Config
     *
     * @return mixed
     */
    public function getConfig()
    {
        return $this->_config;
    }

    /**
     * Sets the Adapter Config
     *
     * @param $config
     */
    public function setConfig($config)
    {
        $this->_config = $config;
    }

    /**
     * Gets the System Config
     *
     * @return mixed
     */
    public function getSystemConfig()
    {
        return $this->_systemconfig;
    }

    /**
     * Sets the System Config
     *
     * @param $config
     */
    public function setSystemConfig($config)
    {
        $this->_systemconfig = $config;
    }

    /**
     * Sets the File Logger
     *
     * @param $fileLogger
     */
    public function setFileLogger($fileLogger)
    {
        $this->_fileLogger = $fileLogger;
    }

    /**
     * Makes the request
     *
     * @param DataObject $data
     * @return mixed
     * @throws Exception
     */
    public function makeRequest(DataObject $data)
    {
        $client = $this->getHttpClient();

        $method = strtoupper($data->getData('method'));
        $endpoint = $data->getData('endpoint');
        $params = $data->getData('params');

        switch ($method) {
            case 'GET':
                $client->get($this->getUrl($endpoint, $params));
                break;
            case 'POST':
                $client->post($this->getUrl($endpoint, $params['get'] ?? []), $params['post'] ?? []);
                break;
            case 'DELETE':
                throw new Exception('Not implemented method DELETE');
            case 'PUT':
                throw new Exception('Not implemented method PUT');
            default:
                throw new Exception('Invalid method');
        }

        return $client;
    }

    /**
     * @return ClientInterface
     */
    private function getHttpClient()
    {
        /** @var ClientInterface $client */
        $client = $this->curlClientFactory->create();

        $username = $this->_systemconfig->getApiUsername();
        $password = $this->_systemconfig->getApiPassword();
        $apiPrefix = $this->_systemconfig->getApiPrefix();
        $baseUrl = $this->_systemconfig->getApiBaseUrl();

        $this->setConfig([
            'baseurl'    => $baseUrl,
            'api_prefix' => $apiPrefix,
            'username'   => $username,
            'password'   => $password,
        ]);

        $client->setHeaders([
            'Content-type' => 'application/json',
            'Accept'       => 'application/json'
        ]);

        $client->setCredentials($username, $password);

        return $client;
    }

    /**
     * @param $endpoint
     * @param array $parameters
     * @return string
     */
    public function getUrl($endpoint, $parameters = [])
    {
        $url = $this->_config['baseurl'] . $this->_config['api_prefix'] . $endpoint;

        if (empty($parameters)) {
            return $url;
        }

        return $url . '?' . http_build_query($parameters);
    }
}
