<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\From\Entity;

use FiloBlu\MappConnector\Model\Config;
use FiloBlu\MappConnector\Model\To\Mapp\SFTP;
use Magento\Eav\Api\AttributeSetRepositoryInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NoSuchEntityException;

/**
 * Class Product
 * @package FiloBlu\MappConnector\Model\From\Entity
 */
class Product extends AbstractEntity implements EntityInterface
{

    //TODO: Get catalog product per store

    /**
     * Entity
     *
     * @var string
     */
    protected $_entity = 'product';

    /**
     * Magento Entity Model
     *
     * @var string
     */
    protected $_entityModel = \Magento\Catalog\Model\Product::class;

    /**
     * Magento Entity Model Id
     *
     * @var string
     */
    protected $_entityModelPrimary = 'entity_id';

    /**
     * Mapp Resource Model
     *
     * @var string
     */
    protected $_entityMappResource = SFTP::class;

    /**
     * Mapp Resource Model Id
     *
     * @var array
     */
    protected $_entityMappPrimary = ['primaryDB' => 'entity_id', 'primaryMapp' => 'entity_id'];

    /**
     * @var \Magento\Catalog\Model\Product
     */
    protected $_model;

    /**
     * @var string
     */
    protected $_xmlConfigPath = 'mappconnector/product_mapping/product';

    /**
     * @var AttributeSetRepositoryInterface
     */
    protected $_attributeSet;

    /**
     * @var Config
     */
    protected $_config;

    /**
     * Product constructor.
     * @param \Magento\Catalog\Model\Product $product
     * @param AttributeSetRepositoryInterface $attributeSet
     * @param Config $config
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        \Magento\Catalog\Model\Product $product,
        AttributeSetRepositoryInterface $attributeSet,
        Config $config,
        ResourceConnection  $resourceConnection
    ) {
        parent::__construct($resourceConnection);
        $this->_model = $product;
        $this->_attributeSet = $attributeSet;
        $this->_config = $config;
    }

    /**
     * @return {@inheritDoc}
     */
    public function getObjectData($primary, $id)
    {
        return $this->_model->getCollection()
            ->addAttributeToSelect('*')
            ->addFieldToFilter($primary, $id)
            ->getFirstItem()
            ->getData();
    }

    /**
     * @param $record
     * @return array @inheritDoc
     * @throws NoSuchEntityException
     */
    public function getDefaultData($record)
    {
        $result = [];

        $result['key'] = $record['sku'];

        //Get Attribute Set Name
        $attributeSetId = $record['attribute_set_id'];
        $attributeSetName = '';
        if (!empty($attributeSetId)) {
            $attributeSet = $this->_attributeSet->get($attributeSetId);
            $attributeSetName = $attributeSet->getAttributeSetName();
        }

        //$result['ProductURI'] = $product->getProductUrl();
        $result['AttributeSet'] = $attributeSetName;
        $result['CurrentPrice'] = $record['price'];
        $result['Name'] = $record['name'];

        return $result;
    }

    /**
     * @param string $type
     * @param string $last_index
     * @return mixed
     */
    public function getAllData($type = 'full', $last_index = '')
    {
        if ($type == 'full') {
            //Full
            return $this->_model->getCollection()
            ->getData();
        } else {
            //Incremental
            $connection = $this->resourceConnection->getConnection();
            $tableName = $this->resourceConnection->getTableName('catalog_product_entity');
            $tableNameJobs =  $this->resourceConnection->getTableName('filoblu_mappconnector_jobs');

            $sql = "SELECT
                      ce.entity_id as entity_id
                    FROM
                      " . $tableName . " AS ce
                    LEFT JOIN
                      " . $tableNameJobs . " AS mj
                    ON
                      mj.entity_id = ce.entity_id
                    AND
                      mj.entity = 'product'
                    WHERE
                      mj.status is null";

            return $connection->fetchAll($sql);
        }
    }
}
