<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Observer;

use Magento\Framework\Event\Observer;

/**
 * Class NewsletterSubscription
 * @package FiloBlu\MappConnector\Observer
 */
class NewsletterSubscription extends \FiloBlu\MappConnector\Observer\AbstractObserver
{
    /**
     * @var string
     */
    protected $_entity = 'subscriber';

    /**
     * @var string
     */
    protected $_type = 'subscriber';

    /**
     * @var string
     */
    protected $_mode = 'rest';

    /**
     * @var string
     */
    protected $_status = 'waiting';

    /**
     * @var \FiloBlu\MappConnector\Model\Jobs
     */
    protected $_jobs;

    /**
     * @var \FiloBlu\MappConnector\Helper\FileLoggerHelper
     */
    protected $_fileLogger;

    /**
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        $skipUpdate = false;
        
        //If Observer is enabled
        if($this->_config->getConfigValue('mappconnector/general/enable_observers')) {
            try {
                $subscriberJob = $this->_jobs->getJobByEntityId($this->_entity, $observer->getData($this->_type)->getSubscriberId());
                if(count($subscriberJob) == 0){
                    $this->_fileLogger->debug('Added new job for ' . $this->_entity . ' entity. ID: ' . $observer->getData($this->_type)->getSubscriberId(),
                        [], __METHOD__ . ':' . __LINE__);
                    $this->_jobs->insertJob([
                        'entity'    => $this->_entity,
                        'entity_id' => $observer->getData($this->_type)->getSubscriberId(),
                        'mode'      => $this->_mode,
                        'status'    => $this->_status,
                        'type'      => 'incremental'
                    ]);

                    $skipUpdate = true;
                }
            }
            catch (\Exception $e) {
                $this->_fileLogger->error('There was an error. The ' . $this->_entity . ' job has not been created (id -> ' . $observer->getData($this->_type)->getSubscriberId() . '). Exception: ' . $e->getMessage());
            }
        }

        //Update status of modified subscriber
        if($this->_config->getConfigValue('mappconnector/general/enable_subscription_change_observers')) {
            if (!$skipUpdate) {
                try {
                    $subscriberJob = $this->_jobs->getJobByEntityId($this->_entity, $observer->getData($this->_type)->getSubscriberId());
                    if (count($subscriberJob) > 0) {
                        $ids = [];
                        foreach ($subscriberJob as $r) {
                            $ids[] = $r['id'];
                        }
                        $this->_jobs->massUpdateJob($ids,'waiting','rest');
                    }
                } catch (\Exception $e) {
                    $this->_fileLogger->error('There was an error. The ' . $this->_entity . ' job has not been updated (id -> ' . $observer->getData($this->_type)->getSubscriberId() . '). Exception: ' . $e->getMessage());
                }
            }
        }:
    }

}